# Copyright (C) 2022  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU Affero General Public License version 3, or any later version
# See top-level LICENSE file for more information

from io import StringIO
from typing import Dict

import pytest

from django.core.management import call_command
from django.db import transaction

from swh.web.mailmap.models import UserMailmap

from .test_mailmap import init_stub_storage_db, populate_mailmap


def call_sync_mailmaps(*args) -> str:
    out = StringIO()
    err = StringIO()

    call_command("sync_mailmaps", *args, stdout=out, stderr=err)

    out.seek(0)
    err.seek(0)
    assert err.read() == ""
    return out.read()


def get_displaynames(postgresql) -> Dict[str, str]:
    with postgresql.cursor() as cur:
        cur.execute(
            "SELECT fullname, displayname FROM person WHERE displayname IS NOT NULL"
        )

        return {bytes(f).decode("utf-8"): bytes(d).decode("utf-8") for (f, d) in cur}


@pytest.mark.django_db(transaction=True)
def test_sync_mailmaps_dry_run(postgresql):
    with transaction.atomic():
        populate_mailmap()

    dsn = init_stub_storage_db(postgresql)

    out = call_sync_mailmaps(dsn)

    assert "(dry run)" in out
    assert "Synced 1 mailmaps to swh.storage database" in out

    assert get_displaynames(postgresql) == {}

    assert (
        UserMailmap.objects.filter(
            from_email_verified=True,
            display_name_activated=True,
            mailmap_last_processing_date__isnull=False,
        ).count()
        == 0
    )


@pytest.mark.django_db(transaction=True)
def test_sync_mailmaps_perform(postgresql):
    with transaction.atomic():
        populate_mailmap()

    dsn = init_stub_storage_db(postgresql)

    out = call_sync_mailmaps("--perform", dsn)

    assert "(dry run)" not in out
    assert "Synced 1 mailmaps to swh.storage database" in out

    expected_displaynames = {
        "Original Name V A <from_emailVA@example.com>": "Display Name V A",
        "Original Name V A 2 <from_emailVA@example.com>": "Display Name V A",
        "Original Name V A 3 <from_emailVA@example.com>": "Display Name V A",
    }

    assert get_displaynames(postgresql) == expected_displaynames

    assert (
        UserMailmap.objects.filter(
            from_email_verified=True,
            display_name_activated=True,
            mailmap_last_processing_date__isnull=False,
        ).count()
        == 1
    )


@pytest.mark.django_db(transaction=True)
def test_sync_mailmaps_with_to_email(postgresql):
    with transaction.atomic():
        populate_mailmap()

    dsn = init_stub_storage_db(postgresql)

    call_sync_mailmaps("--perform", dsn)

    expected_displaynames = {
        "Original Name V A <from_emailVA@example.com>": "Display Name V A",
        "Original Name V A 2 <from_emailVA@example.com>": "Display Name V A",
        "Original Name V A 3 <from_emailVA@example.com>": "Display Name V A",
    }

    assert get_displaynames(postgresql) == expected_displaynames

    # Add a non-valid to_email
    with transaction.atomic():
        for mailmap in UserMailmap.objects.filter(
            from_email_verified=True, display_name_activated=True
        ):
            mailmap.to_email = "to_email@example.com"
            mailmap.save()

    call_sync_mailmaps("--perform", dsn)

    assert get_displaynames(postgresql) == expected_displaynames

    # Verify the relevant to_email
    with transaction.atomic():
        for mailmap in UserMailmap.objects.filter(
            from_email_verified=True, display_name_activated=True
        ):
            mailmap.to_email_verified = True
            mailmap.save()

    call_sync_mailmaps("--perform", dsn)

    new_displayname = "Display Name V A <to_email@example.com>"
    expected_displaynames = {
        "Original Name V A <from_emailVA@example.com>": new_displayname,
        "Original Name V A 2 <from_emailVA@example.com>": new_displayname,
        "Original Name V A 3 <from_emailVA@example.com>": new_displayname,
    }

    assert get_displaynames(postgresql) == expected_displaynames


@pytest.mark.django_db(transaction=True)
def test_sync_mailmaps_disable(postgresql):
    """Check that disabling a mailmap only happens once"""
    with transaction.atomic():
        populate_mailmap()

    dsn = init_stub_storage_db(postgresql)

    # Do the initial mailmap sync
    call_sync_mailmaps("--perform", dsn)

    assert len(get_displaynames(postgresql)) == 3

    updated = 0
    # Disable a display name
    with transaction.atomic():
        # Cannot use update() because `last_update_date` would not be updated
        for mailmap in UserMailmap.objects.filter(
            from_email_verified=True, display_name_activated=True
        ):
            mailmap.display_name_activated = False

            mailmap.save()
            updated += 1

    assert updated == 1

    # Sync mailmaps again
    out = call_sync_mailmaps("--perform", dsn)
    assert "1 mailmaps to disable" in out

    assert get_displaynames(postgresql) == {}

    # Update a displayname by hand
    with postgresql.cursor() as cur:
        cur.execute(
            "UPDATE person SET displayname='Manual Display Name' "
            "WHERE fullname='Original Name V A <from_emailVA@example.com>'"
        )

    expected_displaynames = {
        "Original Name V A <from_emailVA@example.com>": "Manual Display Name"
    }

    assert get_displaynames(postgresql) == expected_displaynames

    # Sync mailmaps one last time. No mailmaps should be disabled
    out = call_sync_mailmaps("--perform", dsn)
    assert "0 mailmaps to disable" in out

    assert get_displaynames(postgresql) == expected_displaynames



# Copyright (C) 2020-2025  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU Affero General Public License version 3, or any later version
# See top-level LICENSE file for more information

import datetime
import random
from urllib.parse import quote, unquote, urlparse

import pytest

from swh.model.from_disk import DentryPerms
from swh.model.hashutil import hash_to_bytes
from swh.model.model import (
    Directory,
    DirectoryEntry,
    Origin,
    OriginVisit,
    OriginVisitStatus,
    Person,
    Release,
    ReleaseTargetType,
    Snapshot,
    SnapshotBranch,
    SnapshotTargetType,
    TimestampWithTimezone,
)
from swh.model.swhids import ObjectType, QualifiedSWHID
from swh.storage.utils import now
from swh.web.browse.snapshot_context import get_snapshot_context
from swh.web.tests.data import random_sha1
from swh.web.tests.helpers import check_html_get_response
from swh.web.utils import reverse
from swh.web.utils.exc import BadInputExc
from swh.web.utils.identifiers import (
    gen_swhid,
    get_qualified_swhid,
    get_swhids_info,
    parse_object_type,
    resolve_swhid,
)
from swh.web.utils.typing import SWHObjectInfo


def test_gen_swhid(content):
    swh_object_type = ObjectType.CONTENT
    sha1_git = content["sha1_git"]

    expected_swhid = "swh:1:cnt:" + sha1_git

    assert gen_swhid(swh_object_type, sha1_git) == expected_swhid

    assert (
        gen_swhid(swh_object_type, sha1_git, metadata={"origin": "test"})
        == expected_swhid + ";origin=test"
    )

    assert (
        gen_swhid(swh_object_type, sha1_git, metadata={"origin": None})
        == expected_swhid
    )

    with pytest.raises(BadInputExc) as e:
        gen_swhid(swh_object_type, "not a valid id")
    assert e.match("Invalid object")


def test_parse_object_type():
    assert parse_object_type("content") == ObjectType.CONTENT
    assert parse_object_type("directory") == ObjectType.DIRECTORY
    assert parse_object_type("revision") == ObjectType.REVISION
    assert parse_object_type("release") == ObjectType.RELEASE
    assert parse_object_type("snapshot") == ObjectType.SNAPSHOT

    with pytest.raises(BadInputExc) as e:
        parse_object_type("foo")
    assert e.match("Invalid swh object type")


def test_resolve_swhid_legacy(content, directory, release, revision, snapshot):
    for obj_type, obj_id in (
        (ObjectType.CONTENT, content["sha1_git"]),
        (ObjectType.DIRECTORY, directory),
        (ObjectType.RELEASE, release),
        (ObjectType.REVISION, revision),
        (ObjectType.SNAPSHOT, snapshot),
    ):
        swhid = gen_swhid(obj_type, obj_id)

        url_args = {}
        if obj_type == ObjectType.CONTENT:
            url_args["query_string"] = f"sha1_git:{obj_id}"
        elif obj_type == ObjectType.SNAPSHOT:
            url_args["snapshot_id"] = obj_id
        else:
            url_args["sha1_git"] = obj_id
        query_params = {"origin_url": "some-origin"}
        browse_url = reverse(
            f"browse-{obj_type.name.lower()}",
            url_args=url_args,
            query_params=query_params,
        )

        for swhid_ in (swhid, swhid.upper()):
            resolved_swhid = resolve_swhid(swhid_, query_params)

            assert isinstance(resolved_swhid["swhid_parsed"], QualifiedSWHID)
            assert str(resolved_swhid["swhid_parsed"]) == swhid
            assert resolved_swhid["browse_url"] == browse_url

    with pytest.raises(BadInputExc, match="'ori' is not a valid ObjectType"):
        resolve_swhid(f"swh:1:ori:{random_sha1()}")


def test_get_qualified_swhid(content, directory, release, revision, snapshot):
    for obj_type, obj_id in (
        (ObjectType.CONTENT, content["sha1_git"]),
        (ObjectType.DIRECTORY, directory),
        (ObjectType.RELEASE, release),
        (ObjectType.REVISION, revision),
        (ObjectType.SNAPSHOT, snapshot),
    ):
        swhid = gen_swhid(obj_type, obj_id)
        for swhid_ in (swhid, swhid.upper()):
            swh_parsed_swhid = get_qualified_swhid(swhid_)
            assert isinstance(swh_parsed_swhid, QualifiedSWHID)
            assert str(swh_parsed_swhid) == swhid.lower()

    with pytest.raises(BadInputExc, match="Error when parsing identifier"):
        get_qualified_swhid("foo")


def test_get_swhid_with_unquoted_white_spaces(content):
    swhid = gen_swhid(
        ObjectType.CONTENT, content["sha1_git"], metadata={"path": "/test/foo bar/baz"}
    )

    swhid_parsed = get_qualified_swhid(swhid)

    assert swhid_parsed.path == b"/test/foo bar/baz"
    assert "path=/test/foo%20bar/baz" in str(swhid_parsed)


def test_get_swhids_info_directory_context(archive_data, directory_with_subdirs):
    swhid = get_swhids_info(
        [
            SWHObjectInfo(
                object_type=ObjectType.DIRECTORY, object_id=directory_with_subdirs
            )
        ],
        snapshot_context=None,
    )[0]
    assert swhid["swhid_with_context"] is None

    # path qualifier should be discarded for a root directory
    swhid = get_swhids_info(
        [
            SWHObjectInfo(
                object_type=ObjectType.DIRECTORY, object_id=directory_with_subdirs
            )
        ],
        snapshot_context=None,
        extra_context={"path": "/"},
    )[0]
    assert swhid["swhid_with_context"] is None

    dir_content = archive_data.directory_ls(directory_with_subdirs)
    dir_subdirs = [e for e in dir_content if e["type"] == "dir"]
    dir_subdir = random.choice(dir_subdirs)
    dir_subdir_path = f'/{dir_subdir["name"]}/'

    dir_subdir_content = archive_data.directory_ls(dir_subdir["target"])
    dir_subdir_files = [e for e in dir_subdir_content if e["type"] == "file"]

    swh_objects_info = [
        SWHObjectInfo(object_type=ObjectType.DIRECTORY, object_id=dir_subdir["target"])
    ]

    extra_context = {
        "root_directory": directory_with_subdirs,
        "path": dir_subdir_path,
    }

    if dir_subdir_files:
        dir_subdir_file = random.choice(dir_subdir_files)
        extra_context["filename"] = dir_subdir_file["name"]
        swh_objects_info.append(
            SWHObjectInfo(
                object_type=ObjectType.CONTENT,
                object_id=dir_subdir_file["checksums"]["sha1_git"],
            )
        )

    swhids = get_swhids_info(
        swh_objects_info,
        snapshot_context=None,
        extra_context=extra_context,
    )

    swhid_lower = swhids[0]["swhid_with_context"]
    swhid_upper = swhid_lower.replace(swhids[0]["swhid"], swhids[0]["swhid"].upper())

    for swhid in (swhid_lower, swhid_upper):
        swhid_dir_parsed = get_qualified_swhid(swhid)

        anchor = gen_swhid(ObjectType.DIRECTORY, directory_with_subdirs)

        assert swhid_dir_parsed.qualifiers() == {
            "anchor": anchor,
            "path": dir_subdir_path,
        }

    if dir_subdir_files:
        swhid_cnt_parsed = get_qualified_swhid(swhids[1]["swhid_with_context"])

        assert swhid_cnt_parsed.qualifiers() == {
            "anchor": anchor,
            "path": f'{dir_subdir_path}{dir_subdir_file["name"]}',
        }


def test_get_swhids_info_revision_context(
    archive_data, revision_with_files_in_target_directory
):
    revision = revision_with_files_in_target_directory
    revision_data = archive_data.revision_get(revision)
    directory = revision_data["directory"]
    dir_content = archive_data.directory_ls(directory)
    dir_entry = random.choice(dir_content)

    swh_objects = [
        SWHObjectInfo(object_type=ObjectType.REVISION, object_id=revision),
        SWHObjectInfo(object_type=ObjectType.DIRECTORY, object_id=directory),
    ]

    extra_context = {"revision": revision, "path": "/"}
    if dir_entry["type"] == "file":
        swh_objects.append(
            SWHObjectInfo(
                object_type=ObjectType.CONTENT,
                object_id=dir_entry["checksums"]["sha1_git"],
            )
        )
        extra_context["filename"] = dir_entry["name"]

    swhids = get_swhids_info(
        swh_objects,
        snapshot_context=None,
        extra_context=extra_context,
    )

    assert swhids[0]["context"] == {}

    swhid_lower = swhids[1]["swhid_with_context"]
    swhid_upper = swhid_lower.replace(swhids[1]["swhid"], swhids[1]["swhid"].upper())

    for swhid in (swhid_lower, swhid_upper):
        swhid_dir_parsed = get_qualified_swhid(swhid)

        anchor = gen_swhid(ObjectType.REVISION, revision)

        assert swhid_dir_parsed.qualifiers() == {
            "anchor": anchor,
        }

    if dir_entry["type"] == "file":
        swhid_cnt_parsed = get_qualified_swhid(swhids[2]["swhid_with_context"])
        assert swhid_cnt_parsed.qualifiers() == {
            "anchor": anchor,
            "path": f'/{dir_entry["name"]}',
        }


def test_get_swhids_info_origin_snapshot_context(
    archive_data, origin_with_multiple_visits
):
    """
    Test SWHIDs with contextual info computation under a variety of origin / snapshot
    browsing contexts.
    """
    origin_url = origin_with_multiple_visits["url"]
    visits = archive_data.origin_visit_get(origin_url)

    for visit in visits:
        snapshot = archive_data.snapshot_get(visit["snapshot"])
        snapshot_id = snapshot["id"]
        branches = {
            k: v["target"]
            for k, v in snapshot["branches"].items()
            if v["target_type"] == "revision"
        }
        releases = {
            k: v["target"]
            for k, v in snapshot["branches"].items()
            if v["target_type"] == "release"
        }
        head_rev_id = archive_data.snapshot_get_head(snapshot)
        head_rev = archive_data.revision_get(head_rev_id)
        root_dir = head_rev["directory"]
        dir_content = archive_data.directory_ls(root_dir)
        dir_files = [e for e in dir_content if e["type"] == "file"]
        dir_file = random.choice(dir_files)
        revision_log = [r["id"] for r in archive_data.revision_log(head_rev_id)]

        branch_name = random.choice(list(branches))
        release = random.choice(list(releases))
        release_data = archive_data.release_get(releases[release])
        release_name = release_data["name"]
        revision_id = random.choice(revision_log)

        for snp_ctx_params, anchor_info in (
            (
                {"snapshot_id": snapshot_id},
                {"anchor_type": ObjectType.REVISION, "anchor_id": head_rev_id},
            ),
            (
                {"snapshot_id": snapshot_id, "branch_name": branch_name},
                {
                    "anchor_type": ObjectType.REVISION,
                    "anchor_id": branches[branch_name],
                },
            ),
            (
                {"snapshot_id": snapshot_id, "release_name": release_name},
                {"anchor_type": ObjectType.RELEASE, "anchor_id": releases[release]},
            ),
            (
                {"snapshot_id": snapshot_id, "revision_id": revision_id},
                {"anchor_type": ObjectType.REVISION, "anchor_id": revision_id},
            ),
            (
                {"origin_url": origin_url, "snapshot_id": snapshot_id},
                {"anchor_type": ObjectType.REVISION, "anchor_id": head_rev_id},
            ),
            (
                {
                    "origin_url": origin_url,
                    "snapshot_id": snapshot_id,
                    "branch_name": branch_name,
                },
                {
                    "anchor_type": ObjectType.REVISION,
                    "anchor_id": branches[branch_name],
                },
            ),
            (
                {
                    "origin_url": origin_url,
                    "snapshot_id": snapshot_id,
                    "release_name": release_name,
                },
                {"anchor_type": ObjectType.RELEASE, "anchor_id": releases[release]},
            ),
            (
                {
                    "origin_url": origin_url,
                    "snapshot_id": snapshot_id,
                    "revision_id": revision_id,
                },
                {"anchor_type": ObjectType.REVISION, "anchor_id": revision_id},
            ),
        ):
            snapshot_context = get_snapshot_context(**snp_ctx_params)

            rev_id = head_rev_id
            if "branch_name" in snp_ctx_params:
                rev_id = branches[branch_name]
            elif "release_name" in snp_ctx_params:
                rev_id = release_data["target"]
            elif "revision_id" in snp_ctx_params:
                rev_id = revision_id

            swh_objects = [
                SWHObjectInfo(
                    object_type=ObjectType.CONTENT,
                    object_id=dir_file["checksums"]["sha1_git"],
                ),
                SWHObjectInfo(object_type=ObjectType.DIRECTORY, object_id=root_dir),
                SWHObjectInfo(object_type=ObjectType.REVISION, object_id=rev_id),
                SWHObjectInfo(object_type=ObjectType.SNAPSHOT, object_id=snapshot_id),
            ]

            if "release_name" in snp_ctx_params:
                swh_objects.append(
                    SWHObjectInfo(
                        object_type=ObjectType.RELEASE, object_id=release_data["id"]
                    )
                )

            swhids = get_swhids_info(
                swh_objects,
                snapshot_context,
                extra_context={"path": "/", "filename": dir_file["name"]},
            )

            swhid_cnt_parsed = get_qualified_swhid(swhids[0]["swhid_with_context"])
            swhid_dir_parsed = get_qualified_swhid(swhids[1]["swhid_with_context"])
            swhid_rev_parsed = get_qualified_swhid(swhids[2]["swhid_with_context"])

            swhid_snp_parsed = get_qualified_swhid(
                swhids[3]["swhid_with_context"] or swhids[3]["swhid"]
            )

            swhid_rel_parsed = None
            if "release_name" in snp_ctx_params:
                swhid_rel_parsed = get_qualified_swhid(swhids[4]["swhid_with_context"])

            anchor = gen_swhid(
                object_type=anchor_info["anchor_type"],
                object_id=anchor_info["anchor_id"],
            )

            snapshot_swhid = gen_swhid(
                object_type=ObjectType.SNAPSHOT, object_id=snapshot_id
            )

            expected_cnt_context = {
                "visit": snapshot_swhid,
                "anchor": anchor,
                "path": f'/{dir_file["name"]}',
            }

            expected_dir_context = {
                "visit": snapshot_swhid,
                "anchor": anchor,
            }

            expected_rev_context = {"visit": snapshot_swhid}

            expected_snp_context = {}

            if "origin_url" in snp_ctx_params:
                expected_cnt_context["origin"] = origin_url
                expected_dir_context["origin"] = origin_url
                expected_rev_context["origin"] = origin_url
                expected_snp_context["origin"] = origin_url

            assert swhid_cnt_parsed.qualifiers() == expected_cnt_context
            assert swhid_dir_parsed.qualifiers() == expected_dir_context
            assert swhid_rev_parsed.qualifiers() == expected_rev_context
            assert swhid_snp_parsed.qualifiers() == expected_snp_context

            if "release_name" in snp_ctx_params:
                assert swhid_rel_parsed.qualifiers() == expected_rev_context


def test_get_swhids_info_characters_and_url_escaping(archive_data, client):
    new_origin_url = "http://example.org/?project=abc;def%"
    sub_directory = Directory(
        entries=(
            DirectoryEntry(
                name=b"bar%",
                type="dir",
                target=Directory(entries=()).id,
                perms=DentryPerms.content,
            ),
        )
    )
    directory = Directory(
        entries=(
            DirectoryEntry(
                name=b"foo;",
                type="dir",
                target=sub_directory.id,
                perms=DentryPerms.content,
            ),
        )
    )
    snapshot = Snapshot(
        branches={
            b"HEAD": SnapshotBranch(
                target=directory.id, target_type=SnapshotTargetType.DIRECTORY
            ),
        },
    )
    archive_data.directory_add([directory, sub_directory])
    archive_data.snapshot_add([snapshot])
    archive_data.origin_add([Origin(url=new_origin_url)])
    origin_visit = archive_data.origin_visit_add(
        [
            OriginVisit(
                origin=new_origin_url,
                date=now(),
                type="git",
            )
        ]
    )[0]
    visit_status = OriginVisitStatus(
        origin=new_origin_url,
        visit=origin_visit.visit,
        date=now(),
        status="full",
        snapshot=snapshot.id,
    )
    archive_data.origin_visit_status_add([visit_status])
    snapshot_context = get_snapshot_context(origin_url=new_origin_url)
    snapshot_context["origin_info"]["url"] = new_origin_url
    path = "/foo;/bar%"

    swhid_info = get_swhids_info(
        [
            SWHObjectInfo(
                object_type=ObjectType.DIRECTORY,
                object_id=snapshot_context["root_directory"],
            )
        ],
        snapshot_context=snapshot_context,
        extra_context={"path": path},
    )[0]

    # check special characters in SWHID have been percent escaped
    assert (
        swhid_info["context"]["origin"] == "http://example.org/?project%3Dabc%3Bdef%25"
    )
    assert swhid_info["context"]["path"] == "/foo%3B/bar%25"

    # check special characters in SWHID URL have been escaped
    swhid_url = swhid_info["swhid_with_context_url"]
    parsed_swhid_url = urlparse(swhid_url)
    assert (
        "origin=http://example.org/%253Fproject%25253Dabc%25253Bdef%252525;"
        in parsed_swhid_url.path
    )
    assert "path=/foo%253B/bar%2525" in parsed_swhid_url.path

    # check that by double unquoting SWHID URL path, we get back on the unquoted SWHID value
    # first unquoting is done by HTTP server, second unquoting by the SWHID parser
    # when processing origin and path qualifiers
    assert unquote(unquote(parsed_swhid_url.path[1:])) == unquote(
        swhid_info["swhid_with_context"]
    )

    # check SWHID can be successfully resolved and browsed
    resp = check_html_get_response(client, swhid_url, status_code=302)
    check_html_get_response(client, resp["location"], status_code=200)


def test_resolve_swhids_snapshot_context(
    client, archive_data, origin_with_multiple_visits
):
    origin_url = origin_with_multiple_visits["url"]
    visits = archive_data.origin_visit_get(origin_url)
    visit = random.choice(visits)
    snapshot = archive_data.snapshot_get(visit["snapshot"])
    head_rev_id = archive_data.snapshot_get_head(snapshot)

    branch_info = None
    release_info = None
    for branch_name in sorted(snapshot["branches"]):
        target_type = snapshot["branches"][branch_name]["target_type"]
        target = snapshot["branches"][branch_name]["target"]
        if target_type == "revision" and branch_info is None:
            branch_info = {"name": branch_name, "revision": target}
        elif target_type == "release" and release_info is None:
            release_info = {"name": branch_name, "release": target}
        if branch_info and release_info:
            break

    release_info["name"] = archive_data.release_get(release_info["release"])["name"]

    directory = archive_data.revision_get(branch_info["revision"])["directory"]
    directory_content = archive_data.directory_ls(directory)
    directory_subdirs = [e for e in directory_content if e["type"] == "dir"]
    directory_subdir = None
    if directory_subdirs:
        directory_subdir = random.choice(directory_subdirs)
    directory_files = [e for e in directory_content if e["type"] == "file"]
    directory_file = None
    if directory_files:
        directory_file = random.choice(directory_files)
    random_rev_id = random.choice(archive_data.revision_log(head_rev_id))["id"]

    for snp_ctx_params in (
        {},
        {"branch_name": branch_info["name"]},
        {"release_name": release_info["name"]},
        {"revision_id": random_rev_id},
    ):
        snapshot_context = get_snapshot_context(
            snapshot["id"], origin_url, **snp_ctx_params
        )

        _check_resolved_swhid_browse_url(
            ObjectType.SNAPSHOT, snapshot["id"], snapshot_context
        )

        rev = head_rev_id
        if "branch_name" in snp_ctx_params:
            rev = branch_info["revision"]
        if "revision_id" in snp_ctx_params:
            rev = random_rev_id

        _check_resolved_swhid_browse_url(ObjectType.REVISION, rev, snapshot_context)

        _check_resolved_swhid_browse_url(
            ObjectType.DIRECTORY, directory, snapshot_context, path="/"
        )

        if directory_subdir:
            _check_resolved_swhid_browse_url(
                ObjectType.DIRECTORY,
                directory_subdir["target"],
                snapshot_context,
                path=f"/{directory_subdir['name']}/",
            )

        if directory_file:
            _check_resolved_swhid_browse_url(
                ObjectType.CONTENT,
                directory_file["target"],
                snapshot_context,
                path=f"/{directory_file['name']}",
            )

            _check_resolved_swhid_browse_url(
                ObjectType.CONTENT,
                directory_file["target"],
                snapshot_context,
                path=f"/{directory_file['name']}",
                lines="10",
            )

            _check_resolved_swhid_browse_url(
                ObjectType.CONTENT,
                directory_file["target"],
                snapshot_context,
                path=f"/{directory_file['name']}",
                lines="10-20",
            )


def _check_resolved_swhid_browse_url(
    object_type, object_id, snapshot_context, path=None, lines=None
):
    snapshot_id = snapshot_context["snapshot_id"]
    origin_url = None
    if snapshot_context["origin_info"]:
        origin_url = snapshot_context["origin_info"]["url"]

    obj_context = {}
    query_params = {}

    if origin_url:
        obj_context["origin"] = origin_url
        query_params["origin_url"] = origin_url

    obj_context["visit"] = gen_swhid(ObjectType.SNAPSHOT, snapshot_id)
    query_params["snapshot"] = snapshot_id

    if object_type in (ObjectType.CONTENT, ObjectType.DIRECTORY, ObjectType.REVISION):
        if snapshot_context["release"]:
            obj_context["anchor"] = gen_swhid(
                ObjectType.RELEASE, snapshot_context["release_id"]
            )
            query_params["release"] = snapshot_context["release"]
        else:
            obj_context["anchor"] = gen_swhid(
                ObjectType.REVISION, snapshot_context["revision_id"]
            )
            if object_type != ObjectType.REVISION:
                query_params["revision"] = snapshot_context["revision_id"]

    if path:
        obj_context["path"] = path
        if path != "/":
            if object_type == ObjectType.CONTENT:
                query_params["path"] = path[1:]
            else:
                query_params["path"] = path[1:-1]

    if object_type == ObjectType.DIRECTORY:
        object_id = snapshot_context["root_directory"]

    if lines:
        obj_context["lines"] = lines

    obj_core_swhid = gen_swhid(object_type, object_id)
    obj_swhid_lower = gen_swhid(object_type, object_id, metadata=obj_context)
    obj_swhid_upper = obj_swhid_lower.replace(obj_core_swhid, obj_core_swhid.upper(), 1)

    for obj_swhid in (obj_swhid_lower, obj_swhid_upper):
        obj_swhid_resolved = resolve_swhid(obj_swhid)

        url_args = {"sha1_git": object_id}
        if object_type == ObjectType.CONTENT:
            url_args = {"query_string": f"sha1_git:{object_id}"}
        elif object_type == ObjectType.SNAPSHOT:
            url_args = {"snapshot_id": object_id}

        expected_url = reverse(
            f"browse-{object_type.name.lower()}",
            url_args=url_args,
            query_params=query_params,
        )
        if lines:
            lines_number = lines.split("-")
            expected_url += f"#L{lines_number[0]}"
            if len(lines_number) > 1:
                expected_url += f"-L{lines_number[1]}"

        assert obj_swhid_resolved["browse_url"] == expected_url


def test_resolve_swhid_with_escaped_chars(archive_data, directory):
    origin_url = "http://example.org/?project=abc;"
    archive_data.origin_add([Origin(url=origin_url)])
    origin_swhid_escaped = quote(origin_url, safe="/?:@&")
    origin_swhid_url_escaped = quote(origin_url, safe="/:@;")
    swhid = gen_swhid(
        ObjectType.DIRECTORY, directory, metadata={"origin": origin_swhid_escaped}
    )
    resolved_swhid = resolve_swhid(swhid)
    assert resolved_swhid["swhid_parsed"].origin == origin_swhid_escaped
    assert origin_swhid_url_escaped in resolved_swhid["browse_url"]


def test_resolve_directory_swhid_path_without_trailing_slash(
    archive_data, directory_with_subdirs
):
    dir_content = archive_data.directory_ls(directory_with_subdirs)
    dir_subdirs = [e for e in dir_content if e["type"] == "dir"]
    dir_subdir = random.choice(dir_subdirs)
    dir_subdir_path = dir_subdir["name"]
    anchor = gen_swhid(ObjectType.DIRECTORY, directory_with_subdirs)
    swhid = gen_swhid(
        ObjectType.DIRECTORY,
        dir_subdir["target"],
        metadata={"anchor": anchor, "path": "/" + dir_subdir_path},
    )
    resolved_swhid = resolve_swhid(swhid)
    browse_url = reverse(
        "browse-directory",
        url_args={"sha1_git": directory_with_subdirs},
        query_params={"path": dir_subdir_path},
    )
    assert resolved_swhid["browse_url"] == browse_url


def test_resolve_swhid_with_malformed_origin_url(archive_data, directory):
    origin_url = "http://example.org/project/abc"
    malformed_origin_url = "http:/example.org/project/abc"
    archive_data.origin_add([Origin(url=origin_url)])
    swhid = gen_swhid(
        ObjectType.DIRECTORY, directory, metadata={"origin": malformed_origin_url}
    )
    resolved_swhid = resolve_swhid(swhid)
    assert origin_url in resolved_swhid["browse_url"]


def test_resolve_dir_entry_swhid_with_anchor_revision(
    archive_data, revision_with_files_in_target_directory
):
    revision = revision_with_files_in_target_directory
    revision_data = archive_data.revision_get(revision)
    directory = revision_data["directory"]
    dir_content = archive_data.directory_ls(directory)
    dir_entry = random.choice(dir_content)

    rev_swhid = gen_swhid(ObjectType.REVISION, revision)

    if dir_entry["type"] == "rev":
        return

    if dir_entry["type"] == "file":
        swhid = gen_swhid(
            ObjectType.CONTENT,
            dir_entry["checksums"]["sha1_git"],
            metadata={"anchor": rev_swhid, "path": f"/{dir_entry['name']}"},
        )

    else:
        swhid = gen_swhid(
            ObjectType.DIRECTORY,
            dir_entry["target"],
            metadata={"anchor": rev_swhid, "path": f"/{dir_entry['name']}/"},
        )

    browse_url = reverse(
        "browse-revision",
        url_args={"sha1_git": revision},
        query_params={"path": dir_entry["name"]},
    )

    resolved_swhid = resolve_swhid(swhid)

    assert resolved_swhid["browse_url"] == browse_url


def test_resolve_dir_entry_swhid_with_anchor_directory(
    archive_data, directory_with_subdirs
):
    dir_content = archive_data.directory_ls(directory_with_subdirs)
    dir_entry = random.choice(
        [entry for entry in dir_content if entry["type"] == "dir"]
    )

    dir_swhid = gen_swhid(ObjectType.DIRECTORY, directory_with_subdirs)

    swhid = gen_swhid(
        ObjectType.DIRECTORY,
        dir_entry["target"],
        metadata={"anchor": dir_swhid, "path": f"/{dir_entry['name']}/"},
    )
    browse_url = reverse(
        "browse-directory",
        url_args={"sha1_git": directory_with_subdirs},
        query_params={"path": f"{dir_entry['name']}"},
    )

    resolved_swhid = resolve_swhid(swhid)

    assert resolved_swhid["browse_url"] == browse_url


def test_resolve_file_entry_swhid_with_anchor_directory(
    archive_data, directory_with_files
):
    dir_content = archive_data.directory_ls(directory_with_files)
    file_entry = random.choice(
        [entry for entry in dir_content if entry["type"] == "file"]
    )

    dir_swhid = gen_swhid(ObjectType.DIRECTORY, directory_with_files)

    sha1_git = file_entry["checksums"]["sha1_git"]
    swhid = gen_swhid(
        ObjectType.CONTENT,
        sha1_git,
        metadata={"anchor": dir_swhid, "path": f"/{file_entry['name']}"},
    )
    browse_url = reverse(
        "browse-content",
        url_args={"query_string": f"sha1_git:{sha1_git}"},
        query_params={"path": f"{directory_with_files}/{file_entry['name']}"},
    )

    resolved_swhid = resolve_swhid(swhid)

    assert resolved_swhid["browse_url"] == browse_url


def test_resolve_dir_swhid_with_release_targeting_dir_anchor(archive_data, directory):
    release_dir_target = Release(
        target=hash_to_bytes(directory),
        target_type=ReleaseTargetType.DIRECTORY,
        date=TimestampWithTimezone.from_datetime(
            datetime.datetime(2015, 1, 1, 22, 0, 0, tzinfo=datetime.timezone.utc)
        ),
        author=Person(
            name=b"author name",
            fullname=b"Author Name author@email",
            email=b"author@email",
        ),
        name=b"v0.0.1",
        message=b"some comment on release",
        synthetic=True,
    )
    release_swhid = gen_swhid(ObjectType.RELEASE, release_dir_target.id.hex())

    archive_data.release_add([release_dir_target])
    snapshot_release_dir_target = Snapshot(
        branches={
            b"v0.0.1": SnapshotBranch(
                target=release_dir_target.id, target_type=SnapshotTargetType.RELEASE
            )
        }
    )
    archive_data.snapshot_add([snapshot_release_dir_target])
    snapshot_release_dir_target_swhid = gen_swhid(
        ObjectType.SNAPSHOT, snapshot_release_dir_target.id.hex()
    )

    swhid = gen_swhid(
        ObjectType.DIRECTORY,
        directory,
        metadata={
            "visit": snapshot_release_dir_target_swhid,
            "anchor": release_swhid,
            "path": "foo",
        },
    )
    assert resolve_swhid(swhid)



# Copyright (C) 2022  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU Affero General Public License version 3, or any later version
# See top-level LICENSE file for more information

import datetime
import json

import pytest

from swh.model.model import Person
from swh.web.mailmap.models import UserMailmap, UserMailmapEvent
from swh.web.tests.helpers import (
    check_api_post_response,
    check_http_get_response,
    check_http_post_response,
)
from swh.web.utils import reverse


@pytest.mark.django_db(transaction=True)
@pytest.mark.parametrize("view_name", ["profile-mailmap-add", "profile-mailmap-update"])
def test_mailmap_endpoints_anonymous_user(api_client, view_name):
    url = reverse(view_name)
    check_api_post_response(api_client, url, status_code=403)


@pytest.mark.django_db(transaction=True)
def test_mailmap_endpoints_user_with_permission(
    api_client, mailmap_user, mailmap_admin
):
    for user, name in ((mailmap_user, "bar"), (mailmap_admin, "baz")):
        UserMailmapEvent.objects.all().delete()

        api_client.force_login(user)

        request_data = {"from_email": f"{name}@example.org", "display_name": name}

        for view_name in ("profile-mailmap-add", "profile-mailmap-update"):
            url = reverse(view_name)
            check_api_post_response(
                api_client,
                url,
                data=request_data,
                status_code=200,
            )

        # FIXME: use check_api_get_responses; currently this crashes without
        # content_type="application/json"
        resp = check_http_get_response(
            api_client,
            reverse("profile-mailmap-list"),
            status_code=200,
            content_type="application/json",
        ).data
        assert len(resp) == 1
        assert resp[0]["from_email"] == f"{name}@example.org"
        assert resp[0]["display_name"] == name

        events = UserMailmapEvent.objects.order_by("timestamp").all()
        assert len(events) == 2
        assert events[0].request_type == "add"
        assert json.loads(events[0].request) == request_data
        assert events[1].request_type == "update"
        assert json.loads(events[1].request) == request_data


@pytest.mark.django_db(transaction=True)
def test_mailmap_add_duplicate(api_client, mailmap_user, mailmap_admin):
    for user, name in ((mailmap_user, "foo"), (mailmap_admin, "bar")):
        api_client.force_login(user)

        check_api_post_response(
            api_client,
            reverse("profile-mailmap-add"),
            data={"from_email": f"{name}@example.org", "display_name": name},
            status_code=200,
        )
        check_api_post_response(
            api_client,
            reverse("profile-mailmap-add"),
            data={"from_email": f"{name}@example.org", "display_name": name},
            status_code=400,
        )


@pytest.mark.django_db(transaction=True)
def test_mailmap_add_full(api_client, mailmap_user, mailmap_admin):
    for user, name in ((mailmap_user, "foo"), (mailmap_admin, "bar")):
        api_client.force_login(user)

        UserMailmapEvent.objects.all().delete()

        request_data = {
            "from_email": f"{name}@example.org",
            "from_email_verified": True,
            "from_email_verification_request_date": "2021-02-07T14:04:15Z",
            "display_name": name,
            "display_name_activated": True,
            "to_email": "baz@example.org",
            "to_email_verified": True,
            "to_email_verification_request_date": "2021-02-07T15:54:59Z",
        }

        check_api_post_response(
            api_client,
            reverse("profile-mailmap-add"),
            data=request_data,
            status_code=200,
        )

        resp = check_http_get_response(
            api_client,
            reverse("profile-mailmap-list"),
            status_code=200,
            content_type="application/json",
        ).data
        assert len(resp) == 1
        assert resp[0].items() >= request_data.items()

        events = UserMailmapEvent.objects.all()
        assert len(events) == 1
        assert events[0].request_type == "add"
        assert json.loads(events[0].request) == request_data
        assert events[0].successful


@pytest.mark.django_db(transaction=True)
def test_mailmap_endpoints_error_response(api_client, mailmap_user, mailmap_admin):
    for user in (mailmap_user, mailmap_admin):
        api_client.force_login(user)

        UserMailmapEvent.objects.all().delete()

        url = reverse("profile-mailmap-add")
        resp = check_api_post_response(api_client, url, status_code=400)
        assert b"from_email" in resp.content

        url = reverse("profile-mailmap-update")
        resp = check_api_post_response(api_client, url, status_code=400)
        assert b"from_email" in resp.content

        events = UserMailmapEvent.objects.order_by("timestamp").all()
        assert len(events) == 2

        assert events[0].request_type == "add"
        assert json.loads(events[0].request) == {}
        assert not events[0].successful

        assert events[1].request_type == "update"
        assert json.loads(events[1].request) == {}
        assert not events[1].successful


@pytest.mark.django_db(transaction=True)
def test_mailmap_update(api_client, mailmap_user, mailmap_admin):
    for user, name in ((mailmap_user, "foo"), (mailmap_admin, "bar")):
        api_client.force_login(user)

        UserMailmapEvent.objects.all().delete()

        before_add = datetime.datetime.now(tz=datetime.timezone.utc)

        check_api_post_response(
            api_client,
            reverse("profile-mailmap-add"),
            data={
                "from_email": f"{name}1@example.org",
                "display_name": "Display Name 1",
            },
            status_code=200,
        )
        check_api_post_response(
            api_client,
            reverse("profile-mailmap-add"),
            data={
                "from_email": f"{name}2@example.org",
                "display_name": "Display Name 2",
            },
            status_code=200,
        )
        after_add = datetime.datetime.now(tz=datetime.timezone.utc)

        user_id = None if user == mailmap_admin else str(user.id)

        mailmaps = list(
            UserMailmap.objects.filter(user_id=user_id).order_by("from_email").all()
        )
        assert len(mailmaps) == 2, mailmaps

        assert mailmaps[0].from_email == f"{name}1@example.org", mailmaps
        assert mailmaps[0].display_name == "Display Name 1", mailmaps
        assert before_add <= mailmaps[0].last_update_date <= after_add

        assert mailmaps[1].from_email == f"{name}2@example.org", mailmaps
        assert mailmaps[1].display_name == "Display Name 2", mailmaps
        assert before_add <= mailmaps[0].last_update_date <= after_add

        before_update = datetime.datetime.now(tz=datetime.timezone.utc)

        check_api_post_response(
            api_client,
            reverse("profile-mailmap-update"),
            data={
                "from_email": f"{name}1@example.org",
                "display_name": "Display Name 1b",
            },
            status_code=200,
        )

        after_update = datetime.datetime.now(tz=datetime.timezone.utc)

        mailmaps = list(
            UserMailmap.objects.filter(user_id=user_id).order_by("from_email").all()
        )
        assert len(mailmaps) == 2, mailmaps

        assert mailmaps[0].from_email == f"{name}1@example.org", mailmaps
        assert mailmaps[0].display_name == "Display Name 1b", mailmaps
        assert before_update <= mailmaps[0].last_update_date <= after_update

        assert mailmaps[1].from_email == f"{name}2@example.org", mailmaps
        assert mailmaps[1].display_name == "Display Name 2", mailmaps
        assert before_add <= mailmaps[1].last_update_date <= after_add

        events = UserMailmapEvent.objects.order_by("timestamp").all()
        assert len(events) == 3
        assert events[0].request_type == "add"
        assert events[1].request_type == "add"
        assert events[2].request_type == "update"


@pytest.mark.django_db(transaction=True)
def test_mailmap_update_from_email_not_found(api_client, mailmap_admin):
    api_client.force_login(mailmap_admin)
    check_api_post_response(
        api_client,
        reverse("profile-mailmap-update"),
        data={
            "from_email": "invalid@example.org",
            "display_name": "Display Name",
        },
        status_code=404,
    )


NB_MAILMAPS = 20
MM_PER_PAGE = 10


def _create_mailmaps(client):
    mailmaps = []
    for i in range(NB_MAILMAPS):
        resp = check_http_post_response(
            client,
            reverse("profile-mailmap-add"),
            data={
                "from_email": f"user{i:02d}@example.org",
                "display_name": f"User {i:02d}",
            },
            status_code=200,
        )
        mailmaps.append(json.loads(resp.content))
    return mailmaps


@pytest.mark.django_db(transaction=True)
def test_mailmap_list_datatables_no_parameters(client, mailmap_admin):
    client.force_login(mailmap_admin)
    mailmaps = _create_mailmaps(client)

    url = reverse("profile-mailmap-list-datatables")

    resp = check_http_get_response(client, url, status_code=200)
    mailmap_data = json.loads(resp.content)

    assert mailmap_data["recordsTotal"] == NB_MAILMAPS
    assert mailmap_data["recordsFiltered"] == NB_MAILMAPS

    # mailmaps sorted by ascending from_email by default
    for i in range(10):
        assert mailmap_data["data"][i]["from_email"] == mailmaps[i]["from_email"]


@pytest.mark.django_db(transaction=True)
@pytest.mark.parametrize("sort_direction", ["asc", "desc"])
def test_mailmap_list_datatables_ordering(client, mailmap_admin, sort_direction):
    client.force_login(mailmap_admin)
    mailmaps = _create_mailmaps(client)
    mailmaps_sorted = list(sorted(mailmaps, key=lambda d: d["display_name"]))
    all_display_names = [mm["display_name"] for mm in mailmaps_sorted]
    if sort_direction == "desc":
        all_display_names = list(reversed(all_display_names))

    for i in range(NB_MAILMAPS // MM_PER_PAGE):
        url = reverse(
            "profile-mailmap-list-datatables",
            query_params={
                "draw": i,
                "length": MM_PER_PAGE,
                "start": i * MM_PER_PAGE,
                "order[0][column]": 2,
                "order[0][name]": "display_name",
                "columns[2][name]": "display_name",
                "order[0][dir]": sort_direction,
            },
        )

        resp = check_http_get_response(client, url, status_code=200)
        data = json.loads(resp.content)

        assert data["draw"] == i
        assert data["recordsFiltered"] == NB_MAILMAPS
        assert data["recordsTotal"] == NB_MAILMAPS
        assert len(data["data"]) == MM_PER_PAGE

        display_names = [mm["display_name"] for mm in data["data"]]

        expected_display_names = all_display_names[
            i * MM_PER_PAGE : (i + 1) * MM_PER_PAGE
        ]
        assert display_names == expected_display_names


@pytest.mark.django_db(transaction=True)
def test_mailmap_list_datatables_search(client, mailmap_admin):
    client.force_login(mailmap_admin)
    _create_mailmaps(client)

    search_value = "user1"

    url = reverse(
        "profile-mailmap-list-datatables",
        query_params={
            "draw": 1,
            "length": MM_PER_PAGE,
            "start": 0,
            "search[value]": search_value,
        },
    )

    resp = check_http_get_response(client, url, status_code=200)
    data = json.loads(resp.content)

    assert data["draw"] == 1
    assert data["recordsFiltered"] == MM_PER_PAGE
    assert data["recordsTotal"] == NB_MAILMAPS
    assert len(data["data"]) == MM_PER_PAGE

    for mailmap in data["data"]:
        assert search_value in mailmap["from_email"]


def populate_mailmap():
    for verified, activated in (
        (False, False),
        (False, True),
        (True, False),
        (True, True),
    ):
        verified_str = "V" if verified else ""
        activated_str = "A" if activated else ""
        UserMailmap.objects.create(
            from_email=f"from_email{verified_str}{activated_str}@example.com",
            display_name=f"Display Name {verified_str} {activated_str}".strip(),
            from_email_verified=verified,
            display_name_activated=activated,
        )


MAILMAP_KNOWN_FULLNAMES = (
    "Original Name <from_email@example.com>",
    "Original Name V <from_emailV@example.com>",
    "Original Name A <from_emailA@example.com>",
    "Original Name V A <from_emailVA@example.com>",
    "Original Name V A 2 <from_emailVA@example.com>",
    "Original Name V A 3 <from_emailVA@example.com>",
)


MAILMAP_KNOWN_PEOPLE = tuple(
    Person.from_fullname(f.encode()) for f in MAILMAP_KNOWN_FULLNAMES
)


def init_stub_storage_db(postgresql):
    with postgresql.cursor() as cur:
        cur.execute(
            """
            CREATE TABLE person (
              fullname bytea PRIMARY KEY,
              name bytea,
              email bytea,
              displayname bytea
            )
            """
        )
        cur.executemany(
            "INSERT INTO person (fullname, name, email) VALUES (%s, %s, %s)",
            [
                (
                    d["fullname"],
                    d["name"],
                    d["email"],
                )
                for d in (p.to_dict() for p in MAILMAP_KNOWN_PEOPLE)
            ],
        )
        cur.execute("CREATE INDEX ON person (email)")
        postgresql.commit()

    return postgresql.info.dsn



# Copyright (C) 2021-2025  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU Affero General Public License version 3, or any later version
# See top-level LICENSE file for more information

import copy
from datetime import datetime, timezone
from itertools import chain
import os
from random import choices, randint
import uuid

import pytest

from django.conf import settings
from django.utils.html import escape

from swh.scheduler.model import LastVisitStatus, ListedOrigin, OriginVisitStats
from swh.web.archive_coverage.views import (
    deposited_origins,
    legacy_origins,
    listed_origins,
)
from swh.web.tests.django_asserts import assert_contains, assert_not_contains
from swh.web.tests.helpers import check_html_get_response, check_http_get_response
from swh.web.utils import reverse
from swh.web.utils.swh_templatetags import static_path_exists


@pytest.fixture(autouse=True)
def count_origins_mocker(mocker):
    mocker.patch("swh.web.utils.archive.count_origins").return_value = randint(10, 100)


def test_coverage_view_no_metrics(client, swh_scheduler):
    """
    Check coverage view can be rendered when scheduler metrics and deposits
    data are not available.
    """
    url = reverse("swh-coverage")
    check_html_get_response(
        client, url, status_code=200, template_used="archive-coverage.html"
    )


visit_types = ["git", "hg", "svn", "bzr", "cvs"]


def generate_archive_coverage_data(mocker, swh_scheduler):
    """Generate some sample scheduler metrics and some sample deposits
    that will be consumed by the archive coverage view.
    """
    listers = []
    for origins in listed_origins["origins"]:
        # create some instances for each lister
        for instance in range(randint(1, 5)):
            lister = swh_scheduler.get_or_create_lister(
                origins["type"], f"instance-{instance}"
            )
            listers.append(lister)
            # record some sample listed origins
            _origins = []
            origin_visit_stats = []
            for i, visit_type in enumerate(visit_types):
                url = str(uuid.uuid4())
                _origins.append(
                    ListedOrigin(
                        lister_id=lister.id,
                        url=url,
                        visit_type=visit_type,
                        extra_loader_arguments={},
                    )
                )
                # set origin visit stats to some origins
                if i % 2 == 0:
                    now = datetime.now(tz=timezone.utc)
                    origin_visit_stats.append(
                        OriginVisitStats(
                            url=url,
                            visit_type=visit_type,
                            last_successful=now,
                            last_visit=now,
                            last_visit_status=LastVisitStatus.successful,
                            last_snapshot=os.urandom(20),
                        )
                    )
            # send origins data to scheduler
            swh_scheduler.record_listed_origins(_origins)
            swh_scheduler.origin_visit_stats_upsert(origin_visit_stats)

    # compute scheduler metrics
    swh_scheduler.update_metrics()

    # add some sample deposits
    deposits = []
    for origins in deposited_origins["origins"]:
        for _ in range(randint(2, 10)):
            deposits.append(
                {
                    "origin_url": f"https://{origins['search_pattern']}/{uuid.uuid4()}",
                    "status": "done",
                }
            )


def test_coverage_view_with_metrics(client, mocker, swh_scheduler):
    generate_archive_coverage_data(mocker, swh_scheduler)

    # check view gets rendered without errors
    url = reverse("swh-coverage")
    resp = check_html_get_response(
        client, url, status_code=200, template_used="archive-coverage.html"
    )

    # check logos and origins search links are present in the rendered page
    for origins in chain(
        listed_origins["origins"],
        legacy_origins["origins"],
        deposited_origins["origins"],
    ):
        origin_type = origins["type"].lower()
        if static_path_exists(f"img/logos/{origin_type}.png"):
            logo_url = f"{settings.STATIC_URL}img/logos/{origin_type}.png"
            assert_contains(resp, f'src="{logo_url}"')

        origin_visit_types = set()

        if "instances" in origins:
            for visit_types_ in origins["instances"].values():
                origin_visit_types.update(visit_types_.keys())
                for data in visit_types_.values():
                    if data["count"] and data["search_url"]:
                        assert_contains(resp, f'<a href="{escape(data["search_url"])}"')
        else:
            for search_url in origins["search_urls"].values():
                assert_contains(resp, f'<a href="{escape(search_url)}"')

    for visit_type in origin_visit_types:
        assert_contains(resp, f"<td>{visit_type}</td>")

    # check request as in production with cache enabled
    check_http_get_response(client, url, status_code=200, server_name="localhost")


def test_coverage_view_with_focus(client, mocker, swh_scheduler):
    generate_archive_coverage_data(mocker, swh_scheduler)

    origins = (
        listed_origins["origins"]
        + legacy_origins["origins"]
        + deposited_origins["origins"]
    )

    focus = choices([o["type"] for o in origins], k=randint(1, 3))

    # check view gets rendered without errors
    url = reverse("swh-coverage", query_params={"focus": ",".join(focus)})
    resp = check_html_get_response(
        client, url, status_code=200, template_used="archive-coverage.html"
    )

    # check focused elements
    assert_contains(
        resp,
        "swh-coverage-focus",
        count=len([o for o in origins if o["type"] in focus]),
    )

    # check bootstrap cards are expanded
    assert_contains(
        resp,
        'class="collapse show"',
        count=len(origins),
    )


@pytest.fixture
def archive_coverage_data_with_non_visited_origins(mocker, swh_scheduler):
    listers = []
    for i, origins in enumerate(listed_origins["origins"]):
        # create one instances for each lister
        lister = swh_scheduler.get_or_create_lister(
            origins["type"], f"instance-{origins['type']}"
        )
        listers.append(lister)

        if i % 2 == 1 or origins["type"] in ("guix", "nixos"):
            # do not declare origins for lister with odd index
            continue

        _origins = []
        origin_visit_stats = []
        for j, visit_type in enumerate(visit_types):
            url = str(uuid.uuid4())
            _origins.append(
                ListedOrigin(
                    lister_id=lister.id,
                    url=url,
                    visit_type=visit_type,
                    extra_loader_arguments={},
                )
            )
            # do not declare visit for visit type with even index
            if j % 2 != 0:
                now = datetime.now(tz=timezone.utc)
                origin_visit_stats.append(
                    OriginVisitStats(
                        url=url,
                        visit_type=visit_type,
                        last_successful=now,
                        last_visit=now,
                        last_visit_status=LastVisitStatus.successful,
                        last_snapshot=os.urandom(20),
                    )
                )
        # send origins data to scheduler
        swh_scheduler.record_listed_origins(_origins)
        swh_scheduler.origin_visit_stats_upsert(origin_visit_stats)

    # compute scheduler metrics
    swh_scheduler.update_metrics()


def test_coverage_view_filter_out_non_visited_origins(
    client, archive_coverage_data_with_non_visited_origins, mocker
):
    # remove display of legacy origins containing a bitbucket origin
    # type that can make this test flaky
    from swh.web.archive_coverage import views

    mocker.patch.object(views, "legacy_origins", {"origins": []})

    origins = copy.copy(listed_origins)

    # check view gets rendered without errors
    url = reverse("swh-coverage")
    resp = check_html_get_response(
        client,
        url,
        status_code=200,
        template_used="archive-coverage.html",
        server_name="not-web-dev",
    )

    for i, origins in enumerate(origins["origins"]):
        if origins["type"] in ("guix", "nixos"):
            continue
        if i % 2 == 1:
            # counters for lister with odd index should not be displayed
            assert_not_contains(resp, f'id="{origins["type"]}"')
        else:
            # counters for lister with even index should be displayed
            assert_contains(resp, f'id="{origins["type"]}"')
            for j, visit_type in enumerate(visit_types):
                if j % 2 == 0:
                    # counter for visit type with even index should be displayed
                    assert_not_contains(resp, f'id="{origins["type"]}-{visit_type}"')
                else:
                    # counter for visit type with odd index should not be displayed
                    assert_contains(resp, f'id="{origins["type"]}-{visit_type}"')


def test_coverage_view_no_logo_for_origin_type(client, mocker, swh_scheduler):
    no_logo_origin_type = "forge_nologo"

    listed_origins["origins"].append(
        {
            "type": no_logo_origin_type,
            "info_url": "",
            "info": "",
            "search_pattern": {
                "default": "",
            },
        }
    )

    generate_archive_coverage_data(mocker, swh_scheduler)

    url = reverse("swh-coverage")
    resp = check_html_get_response(
        client, url, status_code=200, template_used="archive-coverage.html"
    )

    # check logos or placeholders are present in the rendered page
    for origins in chain(
        listed_origins["origins"],
        legacy_origins["origins"],
        deposited_origins["origins"],
    ):
        origin_type = origins["type"].lower()
        if static_path_exists(f"img/logos/{origin_type}.png"):
            logo_url = f"{settings.STATIC_URL}img/logos/{origin_type}.png"
            assert_contains(resp, f'src="{logo_url}"')
        else:
            assert_contains(
                resp,
                (
                    '<text x="50%" y="50%" dominant-baseline="middle" '
                    f'text-anchor="middle">{origin_type}</text>'
                ),
            )



# Copyright (C) 2017-2025  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU Affero General Public License version 3, or any later version
# See top-level LICENSE file for more information

import re
from typing import Optional

from django import template
from django.contrib.staticfiles.finders import find
from django.utils.html import json_script

from swh.web.save_code_now.origin_save import get_savable_visit_types
from swh.web.utils import rst_to_html
from swh.web.utils.converters import SWHDjangoJSONEncoder

register = template.Library()


@register.filter
def docstring_display(docstring):
    """
    Utility function to htmlize reST-formatted documentation in browsable
    api.
    """
    return rst_to_html(docstring, raw_enabled=True)


@register.filter
def urlize_links_and_mails(text):
    """Utility function for decorating api links in browsable api.

    Args:
        text: whose content matching links should be transformed into
        contextual API or Browse html links.

    Returns
        The text transformed if any link is found.
        The text as is otherwise.

    """
    if 'href="' not in text:
        text = re.sub(r"(http.*)", r'<a href="\1">\1</a>', text)
        mails = set()
        for mail in re.findall(r'([^ <>"]+@[^ <>"]+)', text):
            if not mail.startswith(("http://", "https://")):
                mails.add(mail)
        for mail in mails:
            text = text.replace(mail, f'<a href="mailto:{mail}">{mail}</a>')

    return text


@register.filter
def urlize_header_links(text):
    """Utility function for decorating headers links in browsable api.

    Args:
        text: Text whose content contains Link header value

    Returns:
        The text transformed with html link if any link is found.
        The text as is otherwise.

    """
    ret = re.sub(
        r'<(http[^<>]+)>; rel="([^,]+)"',
        r'<<a href="\1">\1</a>>; rel="\2"\n',
        str(text),
    ).replace("\n,", "\n")
    return ret[:-1]


@register.filter(safe=True)
def swh_json_script(value, element_id):
    """Safely outputs a Python object as JSON, wrapped in a <script> tag,
    ready for use with JavaScript.

    Same as ``json_script`` template filter from django but using a custom
    JSON encoder.
    """
    return json_script(value, element_id, encoder=SWHDjangoJSONEncoder)


@register.filter
def sub(value, arg):
    """Django template filter for subtracting two numbers

    Args:
        value (int/float): the value to subtract from
        arg (int/float): the value to subtract to

    Returns:
        int/float: The subtraction result
    """
    return value - arg


@register.filter
def mul(value, arg):
    """Django template filter for multiplying two numbers

    Args:
        value (int/float): the value to multiply from
        arg (int/float): the value to multiply with

    Returns:
        int/float: The multiplication result
    """
    return value * arg


@register.filter
def key_value(dict, key):
    """Django template filter to get a value in a dictionary.

    Args:
        dict (dict): a dictionary
        key (str): the key to lookup value

    Returns:
        The requested value in the dictionary
    """
    return dict[key]


@register.filter
def visit_type_savable(visit_type: str) -> bool:
    """Django template filter to check if a save request can be
    created for a given visit type.

        Args:
            visit_type: the type of visit

        Returns:
            If the visit type is saveable or not

    """
    return visit_type in get_savable_visit_types()


@register.filter
def split(value, arg):
    """Django template filter to split a string.

    Args:
        value (str): the string to split
        arg (str): the split separator

    Returns:
        list: the split string parts
    """
    return value.split(arg)


@register.filter
def static_path_exists(path: str) -> bool:
    """Django template filter to check a static path exists.

    Args:
        path: static path to check existence.

    Returns:
        Whether the path exists.
    """
    return bool(find(path))


@register.filter
def is_markdown(filename: Optional[str]) -> bool:
    """Django template filter to check if a filename is a
    markdown file.
    """
    return filename is not None and filename.lower().endswith((".md", ".markdown"))


@register.filter
def is_notebook(filename: Optional[str]) -> bool:
    """Django template filter to check if a filename is a
    jupyter notebook.
    """
    return filename is not None and filename.lower().endswith(".ipynb")



# Copyright (C) 2020  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU Affero General Public License version 3, or any later version
# See top-level LICENSE file for more information

from swh.web.tests.helpers import check_api_get_responses
from swh.web.utils import reverse


def test_api_1_ping(api_client):
    url = reverse("api-1-ping")
    rv = check_api_get_responses(api_client, url, status_code=200)
    assert rv.data == "pong"



# Copyright (C) 2024  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU Affero General Public License version 3, or any later version
# See top-level LICENSE file for more information

import pytest

import django

APP_NAME = "swh_web_save_bulk"

MIGRATION_0001 = "0001_initial"
MIGRATION_0002 = "0002_alter_savebulkorigin_origin_url"


def test_migrations_02_alter_savebulkorigin_origin_url(migrator):
    old_state = migrator.apply_initial_migration((APP_NAME, MIGRATION_0001))
    old_model = old_state.apps.get_model(APP_NAME, "SaveBulkOrigin")

    short_url = "https://example.org"
    long_url = "https://example.org/" + "e" * 200

    old_model.objects.create(
        visit_type="git",
        origin_url=short_url,
    )

    with pytest.raises(
        django.db.utils.DataError,
        match="value too long for type character",
    ):
        old_model.objects.create(
            visit_type="git",
            origin_url=long_url,
        )

    new_state = migrator.apply_tested_migration((APP_NAME, MIGRATION_0002))
    new_model = new_state.apps.get_model(APP_NAME, "SaveBulkOrigin")

    new_model.objects.create(
        visit_type="git",
        origin_url=long_url,
    )

    assert {sor.origin_url for sor in new_model.objects.all()} == {short_url, long_url}



# Copyright (C) 2022-2024  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU General Public License version 3, or any later version
# See top-level LICENSE file for more information

from datetime import datetime, timezone

import pytest

from django.core.exceptions import ValidationError

from swh.web.add_forge_now.apps import APP_LABEL
from swh.web.add_forge_now.models import RequestStatus

MIGRATION_0001 = "0001_initial"
MIGRATION_0002 = "0002_authorized_null_comment"
MIGRATION_0003 = "0003_request_submitter_forward_username"
MIGRATION_0005 = "0005_prepare_inbound_email"
MIGRATION_0006 = "0006_request_add_new_fields"
MIGRATION_0007 = "0007_rename_denied_request_status"
MIGRATION_0008 = "0008_turn_request_forge_url_into_url_field"
MIGRATION_0009 = "0009_canonicalise_forge_urls"


def now() -> datetime:
    return datetime.now(tz=timezone.utc)


def test_add_forge_now_initial_migration(migrator):
    """Basic migration test to check the model is fine"""

    state = migrator.apply_tested_migration((APP_LABEL, MIGRATION_0001))
    request = state.apps.get_model(APP_LABEL, "Request")
    request_history = state.apps.get_model(APP_LABEL, "RequestHistory")

    from swh.web.add_forge_now.models import RequestActorRole, RequestStatus

    date_now = now()

    req = request(
        status=RequestStatus.PENDING,
        submitter_name="dudess",
        submitter_email="dudess@orga.org",
        forge_type="cgit",
        forge_url="https://example.org/forge",
        forge_contact_email="forge@//example.org",
        forge_contact_name="forge",
        forge_contact_comment=(
            "Discovered on the main forge homepag, following contact link."
        ),
    )
    req.save()

    assert req.submission_date > date_now

    req_history = request_history(
        request=req,
        text="some comment from the moderator",
        actor="moderator",
        actor_role=RequestActorRole.MODERATOR,
        new_status=None,
    )
    req_history.save()
    assert req_history.date > req.submission_date

    req_history2 = request_history(
        request=req,
        text="some answer from the user",
        actor="user",
        actor_role=RequestActorRole.SUBMITTER,
        new_status=None,
    )
    req_history2.save()
    assert req_history2.date > req_history.date


def test_add_forge_now_allow_no_comment(migrator):
    """Basic migration test to check new model authorized empty comment"""

    from django.db.utils import IntegrityError

    state = migrator.apply_tested_migration((APP_LABEL, MIGRATION_0001))

    def make_request_with_empty_comment(requestModel):
        return requestModel(
            status="PENDING",
            submitter_name="dudess",
            submitter_email="dudess@orga.org",
            forge_type="cgit",
            forge_url="https://example.org/forge",
            forge_contact_email="forge@//example.org",
            forge_contact_name="forge",
            forge_contact_comment=None,
        )

    requestModel = state.apps.get_model(APP_LABEL, "Request")

    req = make_request_with_empty_comment(requestModel)
    with pytest.raises(IntegrityError, match="violates not-null constraint"):
        req.save()

    state = migrator.apply_tested_migration((APP_LABEL, MIGRATION_0002))
    requestModel2 = state.apps.get_model(APP_LABEL, "Request")

    req2 = make_request_with_empty_comment(requestModel2)
    req2.save()


def test_add_forge_now_store_submitter_forward_username(migrator):
    state = migrator.apply_tested_migration((APP_LABEL, MIGRATION_0002))
    requestModel = state.apps.get_model(APP_LABEL, "Request")
    assert not hasattr(requestModel, "submitter_forward_username")

    state = migrator.apply_tested_migration((APP_LABEL, MIGRATION_0003))
    requestModel2 = state.apps.get_model(APP_LABEL, "Request")

    assert hasattr(requestModel2, "submitter_forward_username")


def test_add_forge_now_add_new_fields_to_request(migrator):
    state = migrator.apply_tested_migration((APP_LABEL, MIGRATION_0005))
    Request = state.apps.get_model(APP_LABEL, "Request")
    RequestHistory = state.apps.get_model(APP_LABEL, "RequestHistory")
    assert not hasattr(Request, "last_moderator")
    assert not hasattr(Request, "last_modified_date")

    from swh.web.add_forge_now.models import RequestActorRole, RequestStatus

    req = Request(
        status=RequestStatus.PENDING,
        submitter_name="dudess",
        submitter_email="dudess@orga.org",
        forge_type="cgit",
        forge_url="https://example.org/forge",
        forge_contact_email="forge@//example.org",
        forge_contact_name="forge",
        forge_contact_comment=(
            "Discovered on the main forge homepag, following contact link."
        ),
    )
    req.save()

    req_history = RequestHistory(
        request=req,
        text="some comment from the submitter",
        actor="submitter",
        actor_role=RequestActorRole.SUBMITTER.name,
        new_status=None,
    )
    req_history.save()

    req_history = RequestHistory(
        request=req,
        text="some comment from the moderator",
        actor="moderator",
        actor_role=RequestActorRole.MODERATOR.name,
        new_status=None,
    )
    req_history.save()

    state = migrator.apply_tested_migration((APP_LABEL, MIGRATION_0006))
    Request = state.apps.get_model(APP_LABEL, "Request")
    RequestHistory = state.apps.get_model(APP_LABEL, "RequestHistory")

    assert hasattr(Request, "last_moderator")
    assert hasattr(Request, "last_modified_date")

    for request in Request.objects.all():
        history = RequestHistory.objects.filter(request=request)
        history = history.order_by("id")
        assert request.last_modified_date == history.last().date
        assert request.last_moderator == history.last().actor


def test_add_forge_now_denied_status_renamed_to_unsuccesful(migrator):
    state = migrator.apply_tested_migration((APP_LABEL, MIGRATION_0006))
    Request = state.apps.get_model(APP_LABEL, "Request")

    from swh.web.add_forge_now.models import RequestStatus

    req = Request(
        status=RequestStatus.UNSUCCESSFUL.name,
        submitter_name="dudess",
        submitter_email="dudess@orga.org",
        forge_type="cgit",
        forge_url="https://example.org/forge",
        forge_contact_email="forge@example.org",
        forge_contact_name="forge",
        forge_contact_comment=(
            "Discovered on the main forge homepag, following contact link."
        ),
        last_modified_date=datetime.now(timezone.utc),
    )
    with pytest.raises(ValidationError):
        req.clean_fields()

    state = migrator.apply_tested_migration((APP_LABEL, MIGRATION_0007))
    Request = state.apps.get_model(APP_LABEL, "Request")

    req = Request(
        status=RequestStatus.UNSUCCESSFUL.name,
        submitter_name="dudess",
        submitter_email="dudess@orga.org",
        forge_type="cgit",
        forge_url="https://example.org/forge",
        forge_contact_email="forge@example.org",
        forge_contact_name="forge",
        forge_contact_comment=(
            "Discovered on the main forge homepag, following contact link."
        ),
        last_modified_date=datetime.now(timezone.utc),
    )
    req.clean_fields()


def test_add_forge_now_url_validation(migrator):
    state = migrator.apply_tested_migration((APP_LABEL, MIGRATION_0007))
    Request = state.apps.get_model(APP_LABEL, "Request")

    from swh.web.add_forge_now.models import RequestStatus

    request = Request(
        status=RequestStatus.PENDING.name,
        submitter_name="dudess",
        submitter_email="dudess@orga.org",
        forge_type="cgit",
        forge_url="foo",
        forge_contact_email="forge@example.org",
        forge_contact_name="forge",
        forge_contact_comment="bar",
        last_modified_date=datetime.now(timezone.utc),
    )
    request.clean_fields()

    state = migrator.apply_tested_migration((APP_LABEL, MIGRATION_0008))
    Request = state.apps.get_model(APP_LABEL, "Request")

    request = Request(
        status=RequestStatus.PENDING.name,
        submitter_name="johndoe",
        submitter_email="johndoe@example.org",
        forge_type="cgit",
        forge_url="foobar",
        forge_contact_email="forge@example.org",
        forge_contact_name="forge",
        forge_contact_comment="bar",
        last_modified_date=datetime.now(timezone.utc),
    )
    with pytest.raises(ValidationError, match="Enter a valid URL."):
        request.clean_fields()


def test_add_forge_now_canonicalise_forge_urls(migrator):
    state = migrator.apply_tested_migration((APP_LABEL, MIGRATION_0008))
    Request = state.apps.get_model(APP_LABEL, "Request")

    forge_data = [
        ("gitlab", "https://git.codelinaro.org/explore/"),  # to canonicalise
        ("gitea", "https://code-repo.d4science.org/explore/repos"),  # to canonicalise
        ("gitlab", "https://gitlab.example.org/git/"),  # URL must stay the same
    ]

    for forge_type, forge_url in forge_data:
        Request(
            status=RequestStatus.PENDING.name,
            submitter_name="johndoe",
            submitter_email="johndoe@example.org",
            forge_type=forge_type,
            forge_url=forge_url,
            forge_contact_email="forge@example.org",
            forge_contact_name="forge",
            forge_contact_comment="bar",
            last_modified_date=datetime.now(timezone.utc),
        ).save()

    state = migrator.apply_tested_migration((APP_LABEL, MIGRATION_0009))
    Request = state.apps.get_model(APP_LABEL, "Request")

    assert {request.forge_url for request in Request.objects.all()} == {
        "https://git.codelinaro.org/",
        "https://code-repo.d4science.org/",
        "https://gitlab.example.org/git/",
    }



# Copyright (C) 2026  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU Affero General Public License version 3, or any later version
# See top-level LICENSE file for more information

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("swh_web_save_code_now", "0016_alter_saveoriginrequest_origin_url"),
    ]

    operations = [
        migrations.AlterField(
            model_name="saveoriginrequest",
            name="note",
            field=models.TextField(blank=True, default=""),
        ),
        migrations.AlterField(
            model_name="saveoriginrequest",
            name="snapshot_swhid",
            field=models.CharField(blank=True, default="", max_length=200),
        ),
        migrations.AlterField(
            model_name="saveoriginrequest",
            name="user_ids",
            field=models.TextField(blank=True, default=""),
        ),
        migrations.AlterField(
            model_name="saveoriginrequest",
            name="webhook_origin",
            field=models.CharField(blank=True, default="", max_length=200),
        ),
    ]



# Copyright (C) 2017-2026  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU Affero General Public License version 3, or any later version
# See top-level LICENSE file for more information


"""
Django common settings for swh-web.
"""

from importlib.metadata import version
from importlib.util import find_spec
import os
from typing import Any, Dict
import warnings

from django.utils import encoding

from swh.web.config import get_config, oidc_enabled

# Fix django-js-reverse 0.9.1 compatibility with django 4.x
# TODO: Remove that hack once a new django-js-reverse release
# is available on PyPI
if not hasattr(encoding, "force_text"):
    setattr(encoding, "force_text", encoding.force_str)

swh_web_config = get_config()

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
PROJECT_DIR = os.path.dirname(os.path.abspath(__file__))

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.11/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = swh_web_config["secret_key"]
SECRET_KEY_FALLBACKS = swh_web_config["secret_key_fallbacks"]

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = swh_web_config["debug"]
DEBUG_PROPAGATE_EXCEPTIONS = swh_web_config["debug"]

ALLOWED_HOSTS = ["127.0.0.1", "localhost"] + swh_web_config["allowed_hosts"]
UNAUTHENTICATED_HOSTS = ["127.0.0.1", "localhost"] + swh_web_config[
    "unauthenticated_api_hosts"
]

# Applications definition

SWH_BASE_DJANGO_APPS = [
    "swh.web.api",
    "swh.web.auth",
    "swh.web.browse",
    "swh.web.tests",
    "swh.web.utils",
    "swh.web.webapp",
]
SWH_EXTRA_DJANGO_APPS = [
    app
    for app in swh_web_config["swh_extra_django_apps"]
    if app not in SWH_BASE_DJANGO_APPS and app != "swh.web.banners"
]

SWH_DJANGO_APPS = SWH_BASE_DJANGO_APPS + SWH_EXTRA_DJANGO_APPS

SWH_MIRROR_CONFIG = swh_web_config.get("mirror_config", {})
if SWH_MIRROR_CONFIG:
    assert "partner_name" in SWH_MIRROR_CONFIG
    assert "static_path" in SWH_MIRROR_CONFIG
    assert "partner_logo_static_path" in SWH_MIRROR_CONFIG
    assert "templates_path" in SWH_MIRROR_CONFIG

INSTALLED_APPS = [
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.humanize",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "rest_framework",
    "webpack_loader",
    "django_js_reverse",
    "corsheaders",
    "django_bootstrap5",
] + SWH_DJANGO_APPS

MIDDLEWARE = [
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "corsheaders.middleware.CorsMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
    "swh.web.utils.middlewares.ThrottlingHeadersMiddleware",
    "swh.web.utils.middlewares.ExceptionMiddleware",
]

# Compress all assets (static ones and dynamically generated html)
# served by django in a local development environment context.
# In a production environment, assets compression will be directly
# handled by web servers like apache or nginx.
if swh_web_config["serve_assets"]:
    MIDDLEWARE.insert(0, "django.middleware.gzip.GZipMiddleware")

ROOT_URLCONF = "swh.web.urls"

SWH_APP_TEMPLATES = [os.path.join(PROJECT_DIR, "../templates")]
# Add templates directory from each SWH Django application as even
# if an app is not enabled, others might need some of its templates
apps_dir = os.path.join(os.path.dirname(__file__), "../")
_, apps, _ = next(os.walk(apps_dir))  # type: ignore[assignment]
for app in apps:
    app_templates_dir = os.path.join(apps_dir, app, "templates")
    if os.path.exists(app_templates_dir):
        SWH_APP_TEMPLATES.append(app_templates_dir)

# for mirror version of swh-web, we need access to the Save Code Now templates
# even if the django application is not enabled
if SWH_MIRROR_CONFIG and "swh.web.save_code_now" not in SWH_DJANGO_APPS:
    app_spec = find_spec("swh.web.save_code_now")
    assert app_spec is not None and app_spec.origin is not None
    SWH_APP_TEMPLATES.append(
        os.path.join(os.path.dirname(app_spec.origin), "templates")
    )

if SWH_MIRROR_CONFIG:
    mirror_templates_path = SWH_MIRROR_CONFIG["templates_path"]
    assert os.path.exists(os.path.join(mirror_templates_path, "mirror-homepage.html"))
    assert os.path.exists(os.path.join(mirror_templates_path, "mirror-footer.html"))
    SWH_APP_TEMPLATES.append(SWH_MIRROR_CONFIG["templates_path"])

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": SWH_APP_TEMPLATES,
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
                "swh.web.utils.context_processor",
            ],
            "libraries": {
                "swh_templatetags": "swh.web.utils.swh_templatetags",
            },
            "builtins": [
                "django.templatetags.i18n",
                "django_bootstrap5.templatetags.django_bootstrap5",
            ],
        },
    },
]

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": swh_web_config.get("development_db", ""),
    }
}

# Password validation
# https://docs.djangoproject.com/en/1.11/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        "NAME": "django.contrib.auth.password_validation.UserAttributeSimilarityValidator",  # noqa
    },
    {
        "NAME": "django.contrib.auth.password_validation.MinimumLengthValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.CommonPasswordValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.NumericPasswordValidator",
    },
]


# Internationalization
# https://docs.djangoproject.com/en/1.11/topics/i18n/

LANGUAGE_CODE = "en-us"

TIME_ZONE = "UTC"

USE_I18N = True

USE_L10N = True

USE_TZ = True

# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.11/howto/static-files/

STATIC_URL = "/static/"
# static folder location when swh-web has been installed with pip
STATIC_DIR = os.path.join(PROJECT_DIR, "../static")

if not os.path.exists(STATIC_DIR):
    warnings.warn(
        "Unable to find the static assets directory. Check your installation of swh.web."
    )

STATICFILES_DIRS = [STATIC_DIR]

if "static_path" in SWH_MIRROR_CONFIG:
    STATICFILES_DIRS.append(SWH_MIRROR_CONFIG["static_path"])

INTERNAL_IPS = ["127.0.0.1"]

throttle_rates = {}

http_requests = ["GET", "HEAD", "POST", "PUT", "DELETE", "OPTIONS", "PATCH"]

throttling = swh_web_config["throttling"]
for limiter_scope, limiter_conf in throttling["scopes"].items():
    if "default" in limiter_conf["limiter_rate"]:
        throttle_rates[limiter_scope] = limiter_conf["limiter_rate"]["default"]
    # for backward compatibility
    else:
        throttle_rates[limiter_scope] = limiter_conf["limiter_rate"]
    # register sub scopes specific for HTTP request types
    for http_request in http_requests:
        if http_request in limiter_conf["limiter_rate"]:
            throttle_rates[limiter_scope + "_" + http_request.lower()] = limiter_conf[
                "limiter_rate"
            ][http_request]

REST_FRAMEWORK: Dict[str, Any] = {
    "DEFAULT_RENDERER_CLASSES": (
        "rest_framework.renderers.JSONRenderer",
        "swh.web.api.renderers.YAMLRenderer",
        "rest_framework.renderers.TemplateHTMLRenderer",
    ),
    "DEFAULT_THROTTLE_CLASSES": (
        "swh.web.api.throttling.SwhWebRateThrottle",
        "swh.web.api.throttling.SwhWebUserRateThrottle",
    ),
    "DEFAULT_THROTTLE_RATES": throttle_rates,
    "DEFAULT_AUTHENTICATION_CLASSES": [
        "rest_framework.authentication.SessionAuthentication",
        "swh.auth.django.backends.OIDCBearerTokenAuthentication",
    ],
    "EXCEPTION_HANDLER": "swh.web.api.apiresponse.error_response_handler",
}

LOGGING = {
    "version": 1,
    "disable_existing_loggers": False,
    "formatters": {
        "request": {
            "format": "[%(asctime)s] [%(levelname)s] %(request)s %(status_code)s",
            "datefmt": "%d/%b/%Y %H:%M:%S",
        },
        "simple": {
            "format": "[%(asctime)s] [%(levelname)s] [%(name)s] %(message)s",
            "datefmt": "%d/%b/%Y %H:%M:%S",
        },
    },
    "handlers": {
        "console": {
            "level": "DEBUG",
            "class": "logging.StreamHandler",
            "formatter": "simple",
        },
        "null": {
            "class": "logging.NullHandler",
        },
    },
    "loggers": {
        "django": {
            "handlers": ["console"],
            "level": "DEBUG" if DEBUG else "WARNING",
            "propagate": False,
        },
        "django.request": {
            "handlers": ["console"],
            "level": "DEBUG" if DEBUG else "WARNING",
            "propagate": False,
        },
        "django.db.backends": {"handlers": ["null"], "propagate": False},
        "django.utils.autoreload": {
            "level": "INFO",
        },
        "swh.core.statsd": {
            "level": "INFO",
        },
        "swh.web": {
            "handlers": ["console"],
            "level": "DEBUG" if DEBUG else "WARNING",
        },
        "urllib3": {
            "level": "DEBUG" if DEBUG else "INFO",
        },
    },
}

WEBPACK_LOADER = {
    "DEFAULT": {
        "CACHE": False,
        "BUNDLE_DIR_NAME": "./",
        "STATS_FILE": os.path.join(STATIC_DIR, "webpack-stats.json"),
        "POLL_INTERVAL": 0.1,
        "TIMEOUT": None,
        "IGNORE": [".+\\.hot-update.js", ".+\\.map"],
    }
}

AUTHENTICATION_BACKENDS = [
    "django.contrib.auth.backends.ModelBackend",
]

if not oidc_enabled():
    LOGIN_URL = "login"
    LOGOUT_URL = "logout"
else:
    LOGIN_URL = "oidc-login"
    LOGOUT_URL = "oidc-logout"
    AUTHENTICATION_BACKENDS.append(
        "swh.auth.django.backends.OIDCAuthorizationCodePKCEBackend",
    )
    MIDDLEWARE.insert(
        MIDDLEWARE.index("django.contrib.auth.middleware.AuthenticationMiddleware") + 1,
        "swh.auth.django.middlewares.OIDCSessionExpiredMiddleware",
    )

LOGIN_REDIRECT_URL = "swh-web-homepage"

SESSION_ENGINE = "django.contrib.sessions.backends.cache"

CACHES = {
    "default": {
        "BACKEND": "django.core.cache.backends.locmem.LocMemCache",
        "VERSION": version("swh.web"),
    },
    "rate-limit": {"BACKEND": "django.core.cache.backends.locmem.LocMemCache"},
}

JS_REVERSE_JS_MINIFY = False

CORS_ORIGIN_ALLOW_ALL = True
CORS_URLS_REGEX = r"^/(badge|api)/.*$"

OIDC_SWH_WEB_CLIENT_ID = "swh-web"
SWH_AUTH_SERVER_URL = swh_web_config["keycloak"]["server_url"]
SWH_AUTH_REALM_NAME = swh_web_config["keycloak"]["realm_name"]
SWH_AUTH_CLIENT_ID = OIDC_SWH_WEB_CLIENT_ID
SWH_AUTH_SESSION_EXPIRED_REDIRECT_VIEW = "logout-page"

DEFAULT_AUTO_FIELD = "django.db.models.AutoField"

RATELIMIT_USE_CACHE = "rate-limit"
RATELIMIT_ENABLE = False
# get real client IP address when behind production reverse proxy
RATELIMIT_IP_META_KEY = lambda request: request.META.get(  # noqa
    "HTTP_X_ORIGINAL_FORWARDED_FOR", request.META["REMOTE_ADDR"]
).split(",", maxsplit=1)[0]

DATA_UPLOAD_MAX_MEMORY_SIZE = 10485760  # 10Mb

# XXX Transitional setting that will be removed in Django 6.0
FORMS_URLFIELD_ASSUME_HTTPS = True

EMAIL_BACKEND = swh_web_config["email_setup"]["backend"]
EMAIL_HOST = swh_web_config["email_setup"].get("host")
EMAIL_PORT = swh_web_config["email_setup"].get("port")
EMAIL_HOST_USER = swh_web_config["email_setup"].get("username")
EMAIL_HOST_PASSWORD = swh_web_config["email_setup"].get("password")
EMAIL_USE_TLS = swh_web_config["email_setup"].get("use_tls", False)
EMAIL_USE_SSL = swh_web_config["email_setup"].get("use_ssl", False)
DEFAULT_FROM_EMAIL = swh_web_config["email_setup"].get("default_from_email")

ALTER_SETTINGS = swh_web_config.get("alter_settings", {})

BOOTSTRAP5 = {
    "field_renderers": {
        "default": "swh.web.utils.bootstrap.CustomValidationClassFieldRenderer",
    },
}



# Copyright (C) 2024  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU Affero General Public License version 3, or any later version
# See top-level LICENSE file for more information


from django import forms
from django.core.exceptions import ValidationError
from django.http import HttpResponse, HttpResponseBadRequest
from django.utils.crypto import constant_time_compare
from django.utils.decorators import method_decorator
from django.views.decorators.csrf import csrf_exempt
from django.views.generic.edit import FormMixin, ProcessFormView

from ..config import get_config
from .handle_message import MessageHandler


class InboundEmailForm(forms.Form):
    shared_key = forms.CharField()
    email = forms.FileField()

    def clean_shared_key(self):
        shared_key = self.cleaned_data["shared_key"]
        if not constant_time_compare(
            shared_key, get_config()["inbound_email"]["shared_key"]
        ):
            raise ValidationError("Invalid shared key!")

        return shared_key


class InboundEmailView(FormMixin, ProcessFormView):
    form_class = InboundEmailForm
    success_url = "/"
    http_method_names = ["post", "put"]  # no blank (get) InboundEmailForm view

    def form_invalid(self, form):
        return HttpResponseBadRequest(
            form.errors.as_json(), content_type="application/json"
        )

    def form_valid(self, form):
        message_handled = MessageHandler(
            raw_message=form.files["email"].read(), sender=self.__class__
        ).handle()
        if message_handled:
            response_data = {"content": "Message successfully handled\n", "status": 200}
        else:
            response_data = {"content": "Message could not be handled\n", "status": 400}
        return HttpResponse(**response_data, content_type="text/plain")

    @method_decorator(csrf_exempt)
    def dispatch(self, *args, **kwargs):
        return super().dispatch(*args, **kwargs)



# Copyright (C) 2022  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU Affero General Public License version 3, or any later version
# See top-level LICENSE file for more information

from email.message import EmailMessage
from typing import Type

from ..config import get_config
from ..inbound_email.signals import EmailProcessingStatus
from ..inbound_email.utils import get_message_plaintext, get_pks_from_message
from .apps import APP_LABEL
from .models import Request, RequestActorRole, RequestHistory, RequestStatus


def handle_inbound_message(sender: Type, **kwargs) -> EmailProcessingStatus:
    """Handle inbound email messages for add forge now.

    This uses the from field in the message to set the actor for the new entry in the
    request history. We also unconditionally advance the status of requests in the
    ``PENDING`` or ``WAITING_FOR_FEEDBACK`` states.

    The message source is saved in the request history as an escape hatch if something
    goes wrong during processing.

    """

    message = kwargs["message"]
    assert isinstance(message, EmailMessage)

    base_address = get_config()["add_forge_now"]["email_address"]

    pks = get_pks_from_message(
        salt=APP_LABEL, base_address=base_address, message=message
    )

    if not pks:
        return EmailProcessingStatus.IGNORED

    for pk in pks:
        try:
            request = Request.objects.get(pk=pk)
        except Request.DoesNotExist:
            continue

        message_plaintext = get_message_plaintext(message)
        if message_plaintext:
            history_text = message_plaintext
        else:
            history_text = (
                "Could not parse the message contents, see the original message."
            )

        history_entry = RequestHistory(
            request=request,
            actor=str(message["from"]),
            actor_role=RequestActorRole.EMAIL.name,
            text=history_text,
            message_source=bytes(message),
        )

        new_status = {
            RequestStatus.PENDING: RequestStatus.WAITING_FOR_FEEDBACK,
            RequestStatus.WAITING_FOR_FEEDBACK: RequestStatus.FEEDBACK_TO_HANDLE,
        }.get(RequestStatus[request.status])

        if new_status:
            request.status = history_entry.new_status = new_status.name

        history_entry.save()
        # history_entry must be saved to have history.date set
        request.last_modified_date = history_entry.date
        request.save()

    return EmailProcessingStatus.PROCESSED



# Copyright (C) 2017-2026  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU Affero General Public License version 3, or any later version
# See top-level LICENSE file for more information

from __future__ import annotations

import importlib
import os
from typing import TYPE_CHECKING, Any

from swh.core import config
from swh.web import settings

if TYPE_CHECKING:
    from swh.counters.interface import CountersInterface
    from swh.indexer.storage.interface import IndexerStorageInterface
    from swh.scheduler.interface import SchedulerInterface
    from swh.search.interface import SearchInterface
    from swh.storage.interface import StorageInterface
    from swh.vault.interface import VaultInterface

SETTINGS_DIR = os.path.dirname(settings.__file__)


class ConfigurationError(Exception):
    pass


class SWHWebConfig(dict[str, Any]):
    def __getitem__(self, key: str) -> Any:
        try:
            return super().__getitem__(key)
        except KeyError:
            raise ConfigurationError(f"Missing '{key}' configuration")


DEFAULT_CONFIG = {
    "allowed_hosts": ("list", []),
    "unauthenticated_api_hosts": ("list", []),
    "search_config": (
        "dict",
        {
            "metadata_backend": "swh-search",
        },  # or "swh-search"
    ),
    "debug": ("bool", False),
    "serve_assets": ("bool", False),
    "host": ("string", "127.0.0.1"),
    "port": ("int", 5004),
    "secret_key": ("string", "development key"),
    "secret_key_fallbacks": ("list[string]", []),
    # do not display code highlighting for content > 1MB
    "content_display_max_size": ("int", 5 * 1024 * 1024),
    "snapshot_content_max_size": ("int", 1000),
    "throttling": (
        "dict",
        {
            "cache_uri": None,  # production: memcached as cache (127.0.0.1:11211)
            # development: in-memory cache so None
            "scopes": {
                "swh_api": {
                    "limiter_rate": {"default": "120/h"},
                    "exempted_networks": ["127.0.0.0/8"],
                },
                "swh_api_origin_search": {
                    "limiter_rate": {"default": "10/m"},
                    "exempted_networks": ["127.0.0.0/8"],
                },
                "swh_vault_cooking": {
                    "limiter_rate": {"default": "120/h", "GET": "60/m"},
                    "exempted_networks": ["127.0.0.0/8"],
                },
                "swh_save_origin": {
                    "limiter_rate": {"default": "120/h", "POST": "10/h"},
                    "exempted_networks": ["127.0.0.0/8"],
                },
                "swh_api_origin_visit_latest": {
                    "limiter_rate": {"default": "700/m"},
                    "exempted_networks": ["127.0.0.0/8"],
                },
                "swh_api_metadata_citation": {
                    "limiter_rate": {"default": "60/m"},
                    "exempted_networks": ["127.0.0.0/8"],
                },
            },
        },
    ),
    "development_db": ("string", os.path.join(SETTINGS_DIR, "db.sqlite3")),
    "test_db": ("dict", {"name": "swh-web-test"}),
    "production_db": ("dict", {"name": "swh-web"}),
    "e2e_tests_mode": ("bool", False),
    "client_config": ("dict", {}),
    "keycloak": ("dict", {"server_url": "", "realm_name": ""}),
    "counters_backend": ("string", "swh-storage"),  # or "swh-counters"
    "instance_name": ("str", "archive-test.softwareheritage.org"),
    "give": ("dict", {"public_key": "", "token": ""}),
    "features": ("dict", {"add_forge_now": True}),
    "add_forge_now": (
        "dict",
        {
            "email_address": "add-forge-now@example.com",
            "gitlab_pipeline": {
                "token": "sometoken",
                "trigger_url": "someurl",
            },
        },
    ),
    "swh_extra_django_apps": (
        "list",
        [
            "swh.web.add_forge_now",
            "swh.web.alter",
            "swh.web.admin",
            "swh.web.archive_coverage",
            "swh.web.badges",
            "swh.web.banners",
            "swh.web.deposit",
            "swh.web.inbound_email",
            "swh.web.jslicenses",
            "swh.web.mailmap",
            "swh.web.metrics",
            "swh.web.provenance",
            "swh.web.save_bulk",
            "swh.web.save_code_now",
            "swh.web.save_origin_webhooks",
            "swh.web.vault",
        ],
    ),
    "mirror_config": ("dict", {}),
    "top_bar": (
        "dict",
        {
            "links": {
                "Home": "https://www.softwareheritage.org",
                "Development": "https://gitlab.softwareheritage.org",
                "Documentation": "https://docs.softwareheritage.org",
            },
            "donate_link": "https://www.softwareheritage.org/donate",
        },
    ),
    "matomo": ("dict", {}),
    "show_corner_ribbon": ("bool", False),
    "corner_ribbon_text": ("str", ""),
    "save_code_now_webhook_secret": ("str", ""),
    "inbound_email": ("dict", {"shared_key": "shared_key"}),
    "browse_content_rate_limit": ("dict", {"enabled": True, "rate": "60/m"}),
    "activate_citations_ui": ("bool", False),
    "datatables_max_page_size": ("int", 1000),
    "email_setup": (
        "dict",
        {
            "backend": "django.core.mail.backends.smtp.EmailBackend",
            "host": "smtp",
            "port": 1025,
            "username": "username",
            "password": "password",
            "use_tls": False,
            "use_ssl": False,
            "default_from_email": "no-reply@localhost",
        },
    ),
    # when using keycloak as the user backend we use these email aliases to send
    # notifications, this should be replaced by a proper way of querying emails linked
    # to an alter role
    "alter_settings": (
        "dict",
        {
            "support_mail_alias": "alter-support@localhost",
            "manager_mail_alias": "alter-manager@localhost",
            "legal_mail_alias": "alter-legal@localhost",
            "technical_mail_alias": "alter-technical@localhost",
            "block_disposable_email_domains": False,
        },
    ),
    "browse_max_visits": ("int", 10000),
}

swhweb_config: SWHWebConfig = SWHWebConfig()


def get_config(config_file: str = "web/web") -> SWHWebConfig:
    """Read the configuration file `config_file`.

    If an environment variable SWH_CONFIG_FILENAME is defined, this
    takes precedence over the config_file parameter.

    In any case, update the app with parameters (secret_key, conf)
    and return the parsed configuration as a dict.

    If no configuration file is provided, return a default
    configuration.

    """
    if not swhweb_config:
        config_filename = os.environ.get("SWH_CONFIG_FILENAME")
        if config_filename:
            config_file = config_filename
        cfg = config.load_named_config(config_file, DEFAULT_CONFIG)
        swhweb_config.update(cfg)
    for service, modname in (
        ("search", "search"),
        ("storage", "storage"),
        ("vault", "vault"),
        ("indexer_storage", "indexer.storage"),
        ("scheduler", "scheduler"),
        ("counters", "counters"),
    ):
        if isinstance(swhweb_config.get(service), dict):
            mod = importlib.import_module(f"swh.{modname}")
            getter = getattr(mod, f"get_{service}")
            swhweb_config[service] = getter(**swhweb_config[service])
    return swhweb_config


def oidc_enabled() -> bool:
    try:
        return bool(get_config()["keycloak"]["server_url"])
    except:  # noqa: E722
        return False


def search() -> SearchInterface:
    """Return the current application's search."""
    return get_config()["search"]


def storage() -> StorageInterface:
    """Return the current application's storage."""
    return get_config()["storage"]


def vault() -> VaultInterface:
    """Return the current application's vault."""
    return get_config()["vault"]


def indexer_storage() -> IndexerStorageInterface:
    """Return the current application's indexer storage."""
    return get_config()["indexer_storage"]


def scheduler() -> SchedulerInterface:
    """Return the current application's scheduler."""
    return get_config()["scheduler"]


def counters() -> CountersInterface:
    """Return the current application's counters."""
    return get_config()["counters"]



# Copyright (C) 2021-2023  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU Affero General Public License version 3, or any later version
# See top-level LICENSE file for more information

import requests

from django.shortcuts import render
from django.views.decorators.clickjacking import xframe_options_exempt

from swh.web.config import get_config


@xframe_options_exempt
def fundraising_banner(request):
    config = get_config()
    public_key = config["give"]["public_key"]
    token = config["give"]["token"]
    give_api_forms_url = (
        "https://www.softwareheritage.org/give-api/v1/forms/"
        f"?key={public_key}&token={token}&form=27047"
    )

    donations_goal = 100
    nb_donations = -1

    try:
        fundraising_form = requests.get(give_api_forms_url).json().get("forms", [])
        if fundraising_form:
            nb_donations = int(
                fundraising_form[0]
                .get("stats", {})
                .get("total", {})
                .get("donations", -1)
            )
    except Exception:
        pass

    goal_percent = int(nb_donations / donations_goal * 100)

    lang = request.GET.get("lang")

    return render(
        request,
        "fundraising-banner.html",
        {
            "nb_donations": nb_donations,
            "donations_goal": donations_goal,
            "goal_percent": goal_percent,
            "lang": lang if lang else "en",
            "donation_form_url": (
                "https://www.softwareheritage.org/donations/"
                "help-preserve-sourcecode-2021/"
            ),
        },
    )


@xframe_options_exempt
def hiring_banner(request):
    lang = request.GET.get("lang")

    return render(
        request,
        "banner-iframe.html",
        {"lang": lang if lang else "en", "banner_template": "./hiring-banner.html"},
    )


@xframe_options_exempt
def downtime_banner(request):
    lang = request.GET.get("lang")

    return render(
        request,
        "banner-iframe.html",
        {"lang": lang if lang else "en", "banner_template": "downtime-banner.html"},
    )



# Copyright (C) 2022  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU Affero General Public License version 3, or any later version
# See top-level LICENSE file for more information

from django.apps import AppConfig

APP_LABEL = "swh_web_add_forge_now"


class AddForgeNowConfig(AppConfig):
    name = "swh.web.add_forge_now"
    label = APP_LABEL

    def ready(self):
        from ..inbound_email.signals import email_received
        from .signal_receivers import handle_inbound_message

        email_received.connect(handle_inbound_message)



# Copyright (C) 2025  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU Affero General Public License version 3, or any later version
# See top-level LICENSE file for more information

from bs4 import BeautifulSoup

from swh.web.alter.forms import OriginSearchForm
from swh.web.alter.models import AlterationStatus, OriginOutcome
from swh.web.alter.templatetags.alter_extras import (
    absolute_url,
    bootstrap_field_submit,
    outcome_badge,
    status_badge,
)


def test_absolute_url_outside_request():
    assert absolute_url({}, "abc/def") == "abc/def"


def test_absolute_url_path(rf):
    request = rf.get("/")  # RequestFactory sets `testserver` as the default host
    assert absolute_url({"request": request}, "abc/def") == "http://testserver/abc/def"


def test_absolute_url_route(rf, alteration):
    request = rf.get("/")
    url = absolute_url(
        {"request": request},
        "alteration-admin",
        pk=alteration.pk,
    )
    assert alteration.get_admin_url() in url


def test_outcome_badge():
    for value in OriginOutcome:
        assert "my label" in outcome_badge(value, "my label")
    assert "text-bg-secondary" in outcome_badge("test fallback", "my label")


def test_status_badge():
    for value in AlterationStatus:
        assert "my label" in status_badge(value, "my label")
    assert "text-bg-secondary" in status_badge("test fallback", "my label")


def test_bootstrap_field_submit():
    form = OriginSearchForm()
    html = bootstrap_field_submit(form["query"], "Software Heritage")
    soup = BeautifulSoup(html, "lxml")
    assert soup.find("label", class_="visually-hidden")
    button = soup.find("button", type="submit")
    assert button.text == "Software Heritage"
    assert soup.find("input", class_="form-control")



# Copyright (C) 2026  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU Affero General Public License version 3, or any later version
# See top-level LICENSE file for more information

from typing import Tuple

from rest_framework.request import Request

from swh.web.save_origin_webhooks.generic_receiver import OriginSaveWebhookReceiver


class ForgejoOriginSaveWebhookReceiver(OriginSaveWebhookReceiver):
    FORGE_TYPE = "Forgejo"
    WEBHOOK_GUIDE_URL = "https://forgejo.org/docs/latest/user/webhooks/"
    REPO_TYPES = "git"

    def is_forge_request(self, request: Request) -> bool:
        return f"X-{self.FORGE_TYPE}-Event" in request.headers

    def is_push_event(self, request: Request) -> bool:
        return request.headers[f"X-{self.FORGE_TYPE}-Event"] == "push"

    def extract_repo_info(self, request: Request) -> Tuple[str, str, bool]:
        repo_url = request.data.get("repository", {}).get("clone_url", "")
        private = request.data.get("repository", {}).get("private", False)
        return repo_url, "git", private


api_origin_save_webhook_forgejo = ForgejoOriginSaveWebhookReceiver()



# Copyright (C) 2022  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU Affero General Public License version 3, or any later version
# See top-level LICENSE file for more information


# register Web API endpoints
import swh.web.save_origin_webhooks.bitbucket  # noqa
import swh.web.save_origin_webhooks.forgejo  # noqa
import swh.web.save_origin_webhooks.gitea  # noqa
import swh.web.save_origin_webhooks.github  # noqa
import swh.web.save_origin_webhooks.gitlab  # noqa
import swh.web.save_origin_webhooks.gogs  # noqa
import swh.web.save_origin_webhooks.sourceforge  # noqa

from swh.web.save_origin_webhooks.generic_receiver import (  # isort: skip
    webhooks_api_urls,
)

urlpatterns = webhooks_api_urls.get_url_patterns()



# Copyright (C) 2024  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU Affero General Public License version 3, or any later version
# See top-level LICENSE file for more information

from django.urls import re_path as url

from .views import InboundEmailView

urlpatterns = [
    url(r"^inbound-email/$", InboundEmailView.as_view(), name="process-inbound-email")
]



# Copyright (C) 2017-2022  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU Affero General Public License version 3, or any later version
# See top-level LICENSE file for more information

"""
Django development settings for swh-web.
"""

from django.core.cache import cache

from swh.web.config import get_config

swh_web_config = get_config()

swh_web_config.update(
    {
        "show_corner_ribbon": True,
        "corner_ribbon_text": "Development",
    }
)

from .common import *  # noqa

AUTH_PASSWORD_VALIDATORS = []  # disable any pwd validation mechanism

cache.clear()
